#include <sta/rtos/system/startup.hpp>

#include <sta/rtos/c_api/startup.h>

#include <sta/debug/assert.hpp>
#include <sta/lang.hpp>
#include <sta/rtos/system/can_bus.hpp>
#include <sta/rtos/system/events.hpp>
#include <sta/rtos/system/watchdog.hpp>
#include <sta/devices/stm32/init.hpp>

#include <cmsis_os2.h>


namespace sta
{
	namespace rtos
	{
		// Provide weak implementation to allow overriding
		STA_WEAK
		void startupExtras(void *)
		{}


		void initSystem()
		{
#ifdef STA_RTOS_SYSTEM_EVENTS_ENABLE
			initSystemEvents();
#endif // STA_RTOS_SYSTEM_EVENTS_ENABLE

#ifdef STA_RTOS_WATCHDOG_ENABLE
			initWatchdog();
#endif // STA_RTOS_WATCHDOG_ENABLE

#ifdef STA_RTOS_CAN_BUS_ENABLE
			initCanBus();
#endif // STA_RTOS_CAN_BUS_ENABLE
		}
	} // namespace rtos
} // namespace sta


void startALPAKA(void * arg)
{
	STA_ASSERT_MSG(osKernelGetState() != osKernelInactive, "Cannot call startALPAKA() before osKernelInitialize()");

	// Initialize HAL
	sta::initHAL();

	// Initialize RTOS system resources
	sta::rtos::initSystem();

	// Call further initialization code
	sta::rtos::startupExtras(arg);

	// Wake threads
	sta::rtos::signalStartupEvent();

	// Check if called from thread
	if (osThreadGetId() != nullptr)
	{
		// Terminate current thread
		osThreadExit();
	}
}
